@@ -9,7 +9,7 @@ Copyright 2009      James Cameron <quozl@us.netrek.org>
 Copyright 2010      Ibragimov Rinat <ibragimovrinat@mail.ru>
 Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
 Copyright 2019      Daniel Lublin <daniel@lublin.se>
-Copyright 2019-2022 Daniel T. Borelli <danieltborelli@gmail.com>
+Copyright 2019-2023 Daniel T. Borelli <danieltborelli@gmail.com>
 Copyright 2019      Jade Auer <jade@trashwitch.dev>
 Copyright 2020      Sean Brennan <zettix1@gmail.com>
 Copyright 2021      Christopher R. Nelson <christopher.nelson@languidnights.com>
@@ -18,6 +18,7 @@ Copyright 2021      IFo Hancroft <contact@ifohancroft.com>
 Copyright 2021      Peter Wu <peterwu@hotmail.com>
 Copyright 2021      Wilson Smith <01wsmith+gh@gmail.com>
 Copyright 2022      Zev Weiss <zev@bewilderbeest.net>
+Copyright 2023      NRK <nrk@disroot.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -40,7 +41,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
 
-#include <assert.h>
 #include <err.h>
 #include <errno.h>
 #include <getopt.h>
@@ -51,67 +51,62 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <string.h>
 #include <unistd.h>
 
-#include "config.h"
 #include "note.h"
 #include "options.h"
 #include "scrot.h"
 #include "scrot_selection.h"
 #include "util.h"
 
-#define STR_LEN_MAX_FILENAME(msg, fileName) do {                \
-    if (strlen((fileName)) > MAX_FILENAME) {                    \
-        errx(EXIT_FAILURE, #msg " filename too long, must be "  \
-            "less than %d characters", MAX_FILENAME);           \
-    }                                                           \
-} while(0)
-
-#define checkMaxOutputFileName(fileName) \
-    STR_LEN_MAX_FILENAME(output, (fileName))
-
-#define checkMaxInputFileName(fileName) \
-    STR_LEN_MAX_FILENAME(input, (fileName))
-
-enum {
-    MAX_LEN_WINDOW_CLASS_NAME = 80, //characters
-    MAX_FILENAME = 256, // characters
-    MAX_DISPLAY_NAME = 256, // characters
-};
+#ifndef PACKAGE_NAME
+    #define PACKAGE_NAME "scrot"
+#endif
+#ifndef PACKAGE_VERSION
+    #define PACKAGE_VERSION "unknown"
+#endif
 
 struct ScrotOptions opt = {
     .quality = 75,
+    .compression = 7,
     .lineStyle = LineSolid,
     .lineWidth = 1,
     .lineOpacity = SELECTION_OPACITY_DEFAULT,
-    .lineMode = LINE_MODE_S_CLASSIC,
+    .lineMode = LINE_MODE_CLASSIC,
     .stackDirection = HORIZONTAL,
     .monitor = -1,
+    .windowId = None,
+    .outputFile = "%Y-%m-%d-%H%M%S_$wx$h_scrot.png",
+    .lineColor = "gray",
 };
 
 static void showUsage(void);
 static void showVersion(void);
+static long long optionsParseNumBase(const char *, long long, long long,
+    const char *[static 1], int);
 static void optionsParseThumbnail(char *);
+static char *optionsNameThumbnail(const char *);
 
-/* optionsParseNum: "string to number" function.
+long long optionsParseNum(const char *str, long long min, long long max,
+    const char *errmsg[static 1])
+{
+    return optionsParseNumBase(str, min, max, errmsg, 10);
+}
+
+/* optionsParseNumBase: "string to number" function.
  *
  * Parses the string representation of an integer in str, and simultaneously
- * ensures that it is >= min and <= max.
+ * ensures that it is >= min and <= max. The string representation must be of
+ * base `base`.
  *
  * Returns the integer and sets *errmsg to NULL on success.
  * Returns 0 and sets *errmsg to a pointer to a string containing the
  * reason why the number can't be parsed on error.
- *
- * usage:
- * char *errmsg;
- * unsigned int nonnegative;
- * if ((nonnegative = optionsParseNum(optarg, 0, UINT_MAX, &errmsg)) == NULL)
- *     errx(EXIT_FAILURE, "-n: '%s' is %s", optarg, errmsg);
  */
-long long optionsParseNum(const char *str, long long min, long long max,
-    const char *errmsg[static 1])
+static long long optionsParseNumBase(const char *str, long long min,
+    long long max, const char *errmsg[static 1], int base)
 {
     char *end = NULL;
     long long rval;
-    int saved_errno = errno;
+
     if (str == NULL) {
         *errmsg = "missing";
         return 0;
@@ -119,13 +114,13 @@ long long optionsParseNum(const char *str, long long min, long long max,
     *errmsg = NULL;
 
     errno = 0;
-    rval = strtoll(str, &end, 10);
+    rval = strtoll(str, &end, base);
     if (errno == ERANGE) {
         *errmsg = "not representable";
     } else if (*str == '\0') {
         *errmsg = "the null string";
     } else if (*end != '\0') {
-        *errmsg = "not a number";
+        *errmsg = "not an integer";
     } else if (rval < min) {
         /*
          * rval could be set to 0 due to strtoll() returning error and this
@@ -137,7 +132,6 @@ long long optionsParseNum(const char *str, long long min, long long max,
     } else if (rval > max) {
         *errmsg = "too large";
     }
-    errno = saved_errno;
 
     return (*errmsg ? 0 : rval);
 }
@@ -189,22 +183,18 @@ static void optionsParseSelection(const char *optarg)
     if (!strncmp(value, SELECTION_MODE_S_CAPTURE, SELECTION_MODE_L_CAPTURE)) {
         opt.selection.mode = SELECTION_MODE_CAPTURE;
         return; /* it has no parameter */
-    }
-    else if (!strncmp(value, SELECTION_MODE_S_HIDE, SELECTION_MODE_L_HIDE)) {
+    } else if (!strncmp(value, SELECTION_MODE_S_HIDE, SELECTION_MODE_L_HIDE)) {
         opt.selection.mode = SELECTION_MODE_HIDE;
         value += SELECTION_MODE_L_HIDE;
-    }
-    else if (!strncmp(value, SELECTION_MODE_S_HOLE, SELECTION_MODE_L_HOLE)) {
+    } else if (!strncmp(value, SELECTION_MODE_S_HOLE, SELECTION_MODE_L_HOLE)) {
         opt.selection.mode = SELECTION_MODE_HOLE;
-    }
-    else if (!strncmp(value, SELECTION_MODE_S_BLUR, SELECTION_MODE_L_BLUR)) {
+    } else if (!strncmp(value, SELECTION_MODE_S_BLUR, SELECTION_MODE_L_BLUR)) {
         opt.selection.mode = SELECTION_MODE_BLUR;
-        opt.selection.paramNum = SELECTION_MODE_BLUR_DEFAULT;
+        opt.selection.blur = SELECTION_MODE_BLUR_DEFAULT;
         value += SELECTION_MODE_L_BLUR;
-    }
-    else {
+    } else {
         errx(EXIT_FAILURE, "option --select: Unknown value for suboption '%s'",
-             value);
+            value);
     }
 
     if (opt.selection.mode & SELECTION_MODE_NOT_NEED_PARAM)
@@ -218,15 +208,12 @@ static void optionsParseSelection(const char *optarg)
 
     if (opt.selection.mode == SELECTION_MODE_BLUR) {
         const char *errmsg;
-        opt.selection.paramNum = optionsParseNum(value,
+        opt.selection.blur = optionsParseNum(value,
             SELECTION_MODE_BLUR_MIN, SELECTION_MODE_BLUR_MAX, &errmsg);
         if (errmsg)
             errx(EXIT_FAILURE, "option --select: '%s' is %s", value, errmsg);
     } else { // SELECTION_MODE_HIDE
-
-        checkMaxInputFileName(value);
-
-        opt.selection.paramStr = estrdup(value);
+        opt.selection.fileName = value;
     }
 }
 
@@ -261,9 +248,9 @@ static void optionsParseLine(char *optarg)
                     token[Style]);
             }
 
-            if (!strncmp(value, "dash", 4))
+            if (!strcmp(value, "dash"))
                 opt.lineStyle = LineOnOffDash;
-            else if (!strncmp(value, "solid", 5))
+            else if (!strcmp(value, "solid"))
                 opt.lineStyle = LineSolid;
             else {
                 errx(EXIT_FAILURE, "Unknown value for suboption '%s': %s",
@@ -285,7 +272,7 @@ static void optionsParseLine(char *optarg)
                     token[Color]);
             }
 
-            opt.lineColor = estrdup(value);
+            opt.lineColor = value;
             break;
         case Mode:
             if (!optionsParseIsString(value)) {
@@ -293,23 +280,21 @@ static void optionsParseLine(char *optarg)
                     token[Mode]);
             }
 
-            bool isValidMode = !strncmp(value, LINE_MODE_S_CLASSIC, LINE_MODE_L_CLASSIC);
-
-            isValidMode = isValidMode || !strncmp(value, LINE_MODE_S_EDGE, LINE_MODE_L_EDGE);
-
-            if (!isValidMode) {
+            if (!strcmp(value, "classic"))
+                opt.lineMode = LINE_MODE_CLASSIC;
+            else if (!strcmp(value, "edge"))
+                opt.lineMode = LINE_MODE_EDGE;
+            else {
                 errx(EXIT_FAILURE, "Unknown value for suboption '%s': %s",
                     token[Mode], value);
             }
-
-            opt.lineMode = estrdup(value);
             break;
         case Opacity:
             opt.lineOpacity = optionsParseNum(value,
                 SELECTION_OPACITY_MIN, SELECTION_OPACITY_MAX, &errmsg);
             if (errmsg) {
                 if (value == NULL)
-                        value = "(null)";
+                    value = "(null)";
                 errx(EXIT_FAILURE, "option --line: suboption %s: '%s' is %s",
                     token[Opacity], value, errmsg);
             }
@@ -321,166 +306,114 @@ static void optionsParseLine(char *optarg)
     } /* while */
 }
 
-static void optionsParseWindowClassName(const char *windowClassName)
+static const char *getPathOfStdout(void)
 {
-    assert(windowClassName != NULL);
+    const char *paths[] = { "/dev/stdout", "/dev/fd/1", "/proc/self/fd/1" };
 
-    if (windowClassName[0] != '\0')
-        opt.windowClassName = strndup(windowClassName, MAX_LEN_WINDOW_CLASS_NAME);
-}
-
-static bool accessFileOk(const char *const pathName)
-{
-    errno = 0;
-    return (0 == access(pathName, W_OK));
-}
-
-static char *getPathOfStdout(void)
-{
-    char path[16] = {"/dev/stdout"};
-    const size_t len = sizeof(path);
-
-    if (!accessFileOk(path)) {
-
-        snprintf(path, len, "/dev/fd/%d", STDOUT_FILENO);
-
-        if (!accessFileOk(path)) {
-
-            snprintf(path, len, "/proc/self/fd/%d", STDOUT_FILENO);
-
-            if (!accessFileOk(path)) {
-                // We quit because imlib2 will fail later anyway.
-                err(EXIT_FAILURE, "access to stdout failed");
-            }
-        }
+    for (size_t i = 0; i < ARRAY_COUNT(paths); ++i) {
+        if (access(paths[i], W_OK) == 0)
+            return paths[i];
     }
-    return strndup(path, len);
+    err(EXIT_FAILURE, "access to stdout failed");
 }
 
 void optionsParse(int argc, char *argv[])
 {
-    static char stropts[] = "a:ofipbcd:e:hmq:s::t:uvzn:l:D:k::C:S:F:M:";
-
-    static struct option lopts[] = {
-        /* actions */
-        { "help", no_argument, 0, 'h' },
-        { "version", no_argument, 0, 'v' },
-        { "count", no_argument, 0, 'c' },
-        { "focused", no_argument, 0, 'u' },
-        { "focussed", no_argument, 0, 'u' }, /* macquarie dictionary has both spellings */
-        { "border", no_argument, 0, 'b' },
-        { "multidisp", no_argument, 0, 'm' },
-        { "silent", no_argument, 0, 'z' },
-        { "pointer", no_argument, 0, 'p' },
-        { "ignorekeyboard", no_argument, 0, 'i' },
-        { "freeze", no_argument, 0, 'f' },
-        { "overwrite", no_argument, 0, 'o' },
-        /* toggles */
-        { "stack", optional_argument, 0, 'k' },
-        { "select", optional_argument, 0, 's' },
-        { "thumb", required_argument, 0, 't' },
-        { "delay", required_argument, 0, 'd' },
-        { "quality", required_argument, 0, 'q' },
-        { "exec", required_argument, 0, 'e' },
-        { "autoselect", required_argument, 0, 'a' },
-        { "display", required_argument, 0, 'D' },
-        { "note", required_argument, 0, 'n' },
-        { "line", required_argument, 0, 'l' },
-        { "class", required_argument, 0, 'C' },
-        { "script", required_argument, 0, 'S' },
-        { "file", required_argument, 0, 'F' },
-        { "monitor", required_argument, 0, 'M'},
-        { 0, 0, 0, 0 }
+    enum { /* long opt only */
+        /* ensure these don't collude with single byte opts. */
+        OPT_FORMAT = UCHAR_MAX + 1,
     };
-    int optch = 0, cmdx = 0;
+    static char stropts[] = "a:bC:cD:d:e:F:fhik::l:M:mn:opq:S:s::t:uvw:Z:z";
+    static struct option lopts[] = {
+        {"autoselect",      required_argument,  NULL,   'a'},
+        {"border",          no_argument,        NULL,   'b'},
+        {"class",           required_argument,  NULL,   'C'},
+        {"count",           no_argument,        NULL,   'c'},
+        {"display",         required_argument,  NULL,   'D'},
+        {"delay",           required_argument,  NULL,   'd'},
+        {"exec",            required_argument,  NULL,   'e'},
+        {"file",            required_argument,  NULL,   'F'},
+        {"freeze",          no_argument,        NULL,   'f'},
+        {"help",            no_argument,        NULL,   'h'},
+        {"ignorekeyboard",  no_argument,        NULL,   'i'},
+        {"stack",           optional_argument,  NULL,   'k'},
+        {"line",            required_argument,  NULL,   'l'},
+        {"monitor",         required_argument,  NULL,   'M'},
+        {"multidisp",       no_argument,        NULL,   'm'},
+        {"note",            required_argument,  NULL,   'n'},
+        {"overwrite",       no_argument,        NULL,   'o'},
+        {"pointer",         no_argument,        NULL,   'p'},
+        {"quality",         required_argument,  NULL,   'q'},
+        {"script",          required_argument,  NULL,   'S'},
+        {"select",          optional_argument,  NULL,   's'},
+        {"thumb",           required_argument,  NULL,   't'},
+        {"focused",         no_argument,        NULL,   'u'},
+        /* macquarie dictionary has both spellings */
+        {"focussed",        no_argument,        NULL,   'u'},
+        {"version",         no_argument,        NULL,   'v'},
+        {"window",          required_argument,  NULL,   'w'},
+        {"compression",     required_argument,  NULL,   'Z'},
+        {"silent",          no_argument,        NULL,   'z'},
+        {"format",          required_argument,  NULL, OPT_FORMAT},
+        {0}
+    };
+    int optch;
     const char *errmsg;
+    bool FFlagSet = false;
+    const char *note = NULL;
 
     /* Now to pass some optionarinos */
-    while ((optch = getopt_long(argc, argv, stropts, lopts, &cmdx)) != EOF) {
+    while ((optch = getopt_long(argc, argv, stropts, lopts, NULL)) != -1) {
         switch (optch) {
-        case 0:
-            break;
-        case 'h':
-            showUsage();
-            break;
-        case 'v':
-            showVersion();
+        case 'a':
+            optionsParseAutoselect(optarg);
             break;
         case 'b':
-            opt.border = 1;
+            opt.border = true;
+            break;
+        case 'C':
+            opt.windowClassName = optarg;
+            break;
+        case 'c':
+            opt.countdown = true;
+            break;
+        case 'D':
+            opt.display = optarg;
             break;
         case 'd':
-            opt.delay = optionsParseNum(optarg, 0, INT_MAX, &errmsg);
+            opt.delaySelection = *optarg == 'b';
+            if (opt.delaySelection)
+                ++optarg;
+            /* NOTE: div 1000 so that converting to milliseconds doesn't overflow */
+            opt.delay = optionsParseNum(optarg, 0, INT_MAX/1000, &errmsg);
             if (errmsg) {
                 errx(EXIT_FAILURE, "option --delay: '%s' is %s", optarg,
                     errmsg);
             }
             break;
         case 'e':
-            opt.exec = estrdup(optarg);
+            opt.exec = optarg;
             break;
-        case 'm':
-            opt.multidisp = 1;
-            break;
-        case 'q':
-            opt.quality = optionsParseNum(optarg, 1, 100, &errmsg);
-            if (errmsg) {
-                errx(EXIT_FAILURE, "option --quality: '%s' is %s", optarg,
-                    errmsg);
-            }
-            break;
-        case 's':
-            optionsParseSelection(optarg);
-            break;
-        case 'u':
-            opt.focused = 1;
-            break;
-        case 'c':
-            opt.countdown = 1;
-            break;
-        case 't':
-            optionsParseThumbnail(optarg);
-            break;
-        case 'z':
-            opt.silent = 1;
-            break;
-        case 'p':
-            opt.pointer = 1;
-            break;
-        case 'i':
-            opt.ignoreKeyboard = 1;
+        case 'F':
+            FFlagSet = true;
+            opt.outputFile = optarg;
             break;
         case 'f':
-            opt.freeze = 1;
+            opt.freeze = true;
             break;
-        case 'o':
-            opt.overwrite = 1;
+        case 'h':
+            showUsage();
             break;
-        case 'a':
-            optionsParseAutoselect(optarg);
+        case 'i':
+            opt.ignoreKeyboard = true;
             break;
-        case 'D':
-            optionsParseDisplay(optarg);
-            break;
-        case 'n':
-            optionsParseNote(optarg);
+        case 'k':
+            opt.stack = true;
+            optionsParseStack(optarg);
             break;
         case 'l':
             optionsParseLine(optarg);
             break;
-        case 'k':
-            opt.stack = 1;
-            optionsParseStack(optarg);
-            break;
-        case 'C':
-            optionsParseWindowClassName(optarg);
-            break;
-        case 'S':
-            opt.script = estrdup(optarg);
-            break;
-        case 'F':
-            optionsParseFileName(optarg);
-            break;
         case 'M':
             opt.monitor = optionsParseNum(optarg, 0, INT_MAX, &errmsg);
             if (errmsg) {
@@ -488,74 +421,123 @@ void optionsParse(int argc, char *argv[])
                     errmsg);
             }
             break;
-        case '?':
-            exit(EXIT_FAILURE);
-        default:
+        case 'm':
+            opt.multidisp = true;
             break;
+        case 'n':
+            if (optarg[0] == '\0')
+                errx(EXIT_FAILURE, "Required arguments for --note.");
+            note = optarg;
+            break;
+        case 'o':
+            opt.overwrite = true;
+            break;
+        case 'p':
+            opt.pointer = true;
+            break;
+        case 'q':
+            opt.quality = optionsParseNum(optarg, 1, 100, &errmsg);
+            if (errmsg) {
+                errx(EXIT_FAILURE, "option --quality: '%s' is %s", optarg,
+                    errmsg);
+            }
+            break;
+        case 'S':
+            opt.script = optarg;
+            break;
+        case 's':
+            optionsParseSelection(optarg);
+            break;
+        case 't':
+            optionsParseThumbnail(optarg);
+            break;
+        case 'u':
+            opt.focused = true;
+            break;
+        case 'v':
+            showVersion();
+            break;
+        case 'w':
+            opt.windowId = optionsParseNumBase(optarg, None/*0L*/, LONG_MAX, &errmsg, 0);
+            if (errmsg) {
+                errx(EXIT_FAILURE, "option --window: '%s' is %s", optarg,
+                    errmsg);
+            }
+            break;
+        case 'Z':
+            opt.compression = optionsParseNum(optarg, 0, 9, &errmsg);
+            if (errmsg) {
+                errx(EXIT_FAILURE, "option --compression: '%s' is %s", optarg,
+                    errmsg);
+            }
+            break;
+        case 'z':
+            opt.silent = true;
+            break;
+        case OPT_FORMAT:
+            opt.format = optarg;
+            break;
+        default:
+            exit(EXIT_FAILURE);
         }
     }
+    argv += optind;
 
-    /* Now the leftovers, which must be files */
-    while (optind < argc) {
-        /* If recursive is NOT set, but the only argument is a directory
-           name, we grab all the files in there, but not subdirs */
-        if (!opt.outputFile) {
-            optionsParseFileName(argv[optind++]);
-
-            const bool redirectChar = ( opt.outputFile[0] == '-'
-                                        && opt.outputFile[1] == '\0');
-            if (redirectChar) {
-                free(opt.outputFile);
-                opt.outputFile = getPathOfStdout();
-                opt.overwrite = 1;
-                opt.thumbWorP = 0;
-            }
-        } else
-            warnx("unrecognised option %s", argv[optind++]);
+    if (!FFlagSet && *argv) {
+        opt.outputFile = *argv;
+        argv++;
     }
 
-    /* So that we can safely be called again */
-    optind = 1;
+    for (; *argv; ++argv)
+        warnx("ignoring extraneous non-option argument: %s", *argv);
+
+    if (strcmp(opt.outputFile, "-") == 0) {
+        opt.overwrite = true;
+        opt.thumb = THUMB_DISABLED;
+        opt.outputFile = getPathOfStdout();
+    }
+    if (opt.thumb != THUMB_DISABLED)
+        opt.thumbFile = optionsNameThumbnail(opt.outputFile);
+
+    if (note) {
+        warnx("--note is deprecated. See: "
+            "https://github.com/resurrecting-open-source-projects/scrot/discussions/207");
+        opt.note = estrdup(note); /* TODO: investigate if dup is needed */
+        scrotNoteNew(opt.note);
+    }
 }
 
 static void showUsage(void)
 {
-    fputs(/* Check that everything lines up after any changes. */
-        "usage:  " PACKAGE " [-bcfhimopuvz] [-a X,Y,W,H] [-C NAME] [-D DISPLAY]\n"
-        "              [-d SEC] [-e CMD] [-F FILE] [-k OPT] [-l STYLE] [-M NUM]\n"
-        "              [-n OPTS] [-q NUM] [-S CMD] [-s OPTS] [-t NUM | GEOM] [FILE]\n",
+    fputs("usage:  " /* Check that everything lines up after any changes. */
+        PACKAGE_NAME " [-bcfhimopuvz] [-a X,Y,W,H] [-C NAME] [-D DISPLAY]\n"
+        "              [-d SEC] [-e CMD] [-k OPT] [-l STYLE] [-M NUM] [-q NUM]\n"
+        "              [-s OPTS] [-t % | WxH] [[-F] FILE]\n",
         stdout);
     exit(0);
 }
 
 static void showVersion(void)
 {
-    printf(PACKAGE " version " VERSION "\n");
+    puts(PACKAGE_NAME " version " PACKAGE_VERSION);
     exit(0);
 }
 
-char *optionsNameThumbnail(const char *name)
+static char *optionsNameThumbnail(const char *name)
 {
-    const char *const thumbSuffix = "-thumb";
-    const size_t thumbSuffixLength = 7;
-    const size_t newNameLength = strlen(name) + thumbSuffixLength;
-    char *newName = calloc(1, newNameLength);
-
-    if (!newName)
-        err(EXIT_FAILURE, "Unable to allocate thumbnail");
-
+    const ptrdiff_t nameLength = strlen(name);
+    const char thumbSuffix[] = "-thumb";
+    Stream ret = {0};
     const char *const extension = strrchr(name, '.');
+    const ptrdiff_t baseNameLength = extension ? extension-name : nameLength;
 
-    if (extension) {
-        /* We add one so length includes '\0'*/
-        const ptrdiff_t nameLength = (extension - name) + 1;
-        strlcpy(newName, name, nameLength);
-        strlcat(newName, thumbSuffix, newNameLength);
-        strlcat(newName, extension, newNameLength);
-    } else
-        snprintf(newName, newNameLength, "%s%s", name, thumbSuffix);
+    streamMem(&ret, name, baseNameLength);
+    streamMem(&ret, thumbSuffix, sizeof(thumbSuffix)-1);
+    if (extension)
+        streamMem(&ret, extension, nameLength - baseNameLength);
+    streamChar(&ret, '\0');
 
-    return newName;
+    return ret.buf;
 }
 
 void optionsParseAutoselect(char *optarg)
@@ -581,17 +563,10 @@ void optionsParseAutoselect(char *optarg)
         }
         i++;
 
-        opt.autoselect = 1;
+        opt.autoselect = true;
     }
     if (i < 4)
-            errx(EXIT_FAILURE, "option --autoselect: too few dimensions");
-}
-
-void optionsParseDisplay(char *optarg)
-{
-    opt.display = strndup(optarg, MAX_DISPLAY_NAME);
-    if (!opt.display)
-        err(EXIT_FAILURE, "Unable to allocate display");
+        errx(EXIT_FAILURE, "option --autoselect: too few dimensions");
 }
 
 static void optionsParseThumbnail(char *optarg)
@@ -603,56 +578,27 @@ static void optionsParseThumbnail(char *optarg)
         /* optarg holds the width, height holds the height. */
         *height++ = '\0';
 
-        opt.thumbWorP = optionsParseNum(optarg, 1, INT_MAX, &errmsg);
+        opt.thumb = THUMB_RES;
+        opt.thumbW = optionsParseNum(optarg, 0, INT_MAX, &errmsg);
         if (errmsg) {
             errx(EXIT_FAILURE, "option --thumb: resolution width '%s' is %s",
                 optarg, errmsg);
         }
 
-        opt.thumbH = optionsParseNum(height, 1, INT_MAX, &errmsg);
+        opt.thumbH = optionsParseNum(height, 0, INT_MAX, &errmsg);
         if (errmsg) {
             errx(EXIT_FAILURE, "option --thumb: resolution height '%s' is %s",
                 height, errmsg);
         }
+
+        if (opt.thumbW == 0 && opt.thumbH == 0)
+            errx(EXIT_FAILURE, "option --thumb: both width and height are 0");
     } else { /* optarg is a percentage. */
-        opt.thumbWorP = optionsParseNum(optarg, 1, INT_MAX, &errmsg);
+        opt.thumb = THUMB_PERCENT;
+        opt.thumbPercent = optionsParseNum(optarg, 1, 100, &errmsg);
         if (errmsg) {
             errx(EXIT_FAILURE, "option --thumb: percentage '%s' is %s", optarg,
                 errmsg);
         }
     }
 }
-
-void optionsParseFileName(const char *optarg)
-{
-    checkMaxOutputFileName(optarg);
-    opt.outputFile = estrdup(optarg);
-}
-
-void optionsParseNote(char *optarg)
-{
-    if (opt.note)
-        free(opt.note);
-
-    opt.note = estrdup(optarg);
-
-    if (!opt.note)
-        return;
-
-    if (opt.note[0] == '\0')
-        errx(EXIT_FAILURE, "Required arguments for --note.");
-
-    scrotNoteNew(opt.note);
-}
-
-/*
-Return:
-    0 : It does not match
-    1 : If it matches
-*/
-int optionsCompareWindowClassName(const char *targetClassName)
-{
-    assert(targetClassName != NULL);
-    assert(opt.windowClassName != NULL);
-    return !!(!strncmp(targetClassName, opt.windowClassName, MAX_LEN_WINDOW_CLASS_NAME - 1));
-}
