@@ -6,13 +6,14 @@ Copyright 1999-2000 Tom Gilbert <tom@linuxbrit.co.uk,
 Copyright 2009      James Cameron <quozl@us.netrek.org>
 Copyright 2010      Ibragimov Rinat <ibragimovrinat@mail.ru>
 Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
-Copyright 2019-2022 Daniel T. Borelli <danieltborelli@gmail.com>
+Copyright 2019-2023 Daniel T. Borelli <danieltborelli@gmail.com>
 Copyright 2020      Sean Brennan <zettix1@gmail.com>
 Copyright 2021-2023 Guilherme Janczak <guilherme.janczak@yandex.com>
 Copyright 2021      IFo Hancroft <contact@ifohancroft.com>
 Copyright 2021      Peter Wu <peterwu@hotmail.com>
 Copyright 2021      Wilson Smith <01wsmith+gh@gmail.com>
 Copyright 2022      Zev Weiss <zev@bewilderbeest.net>
+Copyright 2023      NRK <nrk@disroot.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -35,7 +36,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
 
-#pragma once
+#ifndef H_OPTIONS
+#define H_OPTIONS
+
+#include <time.h>
 
 #include "scrot_selection.h"
 
@@ -48,53 +52,53 @@ enum Direction {
 
 struct ScrotOptions {
     int delay;
-    int countdown;
-    int focused;
+    struct timespec delayStart;
     int quality;
-    int border;
-    int silent;
-    int multidisp;
-    /* If true, the --thumb option is in use. */
-    int thumbWorP;
-    /* If thumbH is true, thumbWorP contains a width and thumbH contains a
-     * height. Otherwise, thumbWorP contains a percentage.
-     */
+    int compression;
+    enum thumb { THUMB_DISABLED, THUMB_PERCENT, THUMB_RES } thumb;
+    int thumbPercent;
+    int thumbW;
     int thumbH;
-    int pointer;
-    int ignoreKeyboard;
-    int freeze;
-    int overwrite;
+    enum LineMode lineMode;
     int lineStyle;
     int lineWidth;
     int lineOpacity;
-    int stack;
     enum Direction stackDirection;
-    char *lineColor;
-    char *lineMode;
-    char *outputFile;
+    const char *format;
+    const char *lineColor;
+    const char *outputFile;
     char *thumbFile;
-    char *exec;
-    char *display;
+    const char *exec;
+    const char *display;
     char *note;
-    char *windowClassName;
-    char *script;
-    int autoselect;
+    Window windowId;
+    const char *windowClassName;
+    const char *script;
     int autoselectX;
     int autoselectY;
     int autoselectH;
     int autoselectW;
     SelectionMode selection;
     int monitor;
+    bool delaySelection;
+    bool countdown;
+    bool border;
+    bool silent;
+    bool focused;
+    bool pointer;
+    bool overwrite;
+    bool multidisp;
+    bool freeze;
+    bool ignoreKeyboard;
+    bool stack;
+    bool autoselect;
 };
 
 extern struct ScrotOptions opt;
 
 void optionsParse(int, char **);
-char *optionsNameThumbnail(const char *);
-void optionsParseFileName(const char *);
 void optionsParseAutoselect(char *);
-void optionsParseDisplay(char *);
-void optionsParseNote(char *);
 long long optionsParseNum(const char *, long long, long long,
     const char *[static 1]);
-int optionsCompareWindowClassName(const char *);
+
+#endif /* !defined(H_OPTIONS) */
