@@ -6,11 +6,12 @@ Copyright 1999-2000 Tom Gilbert <tom@linuxbrit.co.uk,
 Copyright 2009      James Cameron <quozl@us.netrek.org>
 Copyright 2010      Ibragimov Rinat <ibragimovrinat@mail.ru>
 Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
-Copyright 2019-2022 Daniel T. Borelli <danieltborelli@gmail.com>
+Copyright 2019-2023 Daniel T. Borelli <danieltborelli@gmail.com>
 Copyright 2019      Jade Auer <jade@trashwitch.dev>
 Copyright 2020      blockparole
 Copyright 2020      Cungsten Tarbide <ctarbide@tuta.io>
 Copyright 2020      Hinigatsu <hinigatsu@protonmail.com>
+Copyright 2020      ideal <idealities@gmail.com>
 Copyright 2020      nothub
 Copyright 2020      Sean Brennan <zettix1@gmail.com>
 Copyright 2021      c0dev0id <sh+github@codevoid.de>
@@ -18,7 +19,7 @@ Copyright 2021      Christopher R. Nelson <christopher.nelson@languidnights.com>
 Copyright 2021-2023 Guilherme Janczak <guilherme.janczak@yandex.com>
 Copyright 2021      IFo Hancroft <contact@ifohancroft.com>
 Copyright 2021      Peter Wu <peterwu@hotmail.com>
-Copyright 2022      NRK <nrk@disroot.org>
+Copyright 2022-2023 NRK <nrk@disroot.org>
 Copyright 2022      Zev Weiss <zev@bewilderbeest.net>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -45,10 +46,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <sys/stat.h>
 #include <sys/wait.h>
 
-#include <assert.h>
 #include <err.h>
 #include <errno.h>
 #include <stdio.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -61,31 +62,39 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <X11/extensions/Xfixes.h>
 #include <X11/extensions/Xinerama.h>
 
-#include "imlib.h"
 #include "note.h"
 #include "options.h"
 #include "scrot.h"
 #include "slist.h"
 #include "util.h"
 
+static void initXAndImlib(const char *, int);
 static void uninitXAndImlib(void);
 static size_t scrotHaveFileExtension(const char *, char **);
 static Imlib_Image scrotGrabFocused(void);
 static void applyFilterIfRequired(void);
-static Bool scrotXEventVisibility(Display *, XEvent *, XPointer);
 static Imlib_Image scrotGrabAutoselect(void);
+static long miliToNanoSec(int);
 static Imlib_Image scrotGrabShotMulti(void);
 static Imlib_Image scrotGrabShotMonitor(void);
 static Imlib_Image scrotGrabStackWindows(void);
 static Imlib_Image scrotGrabShot(void);
 static void scrotCheckIfOverwriteFile(char **);
 static void scrotExecApp(Imlib_Image, struct tm *, char *, char *);
-static char *imPrintf(char *, struct tm *, char *, char *, Imlib_Image);
+static char *imPrintf(const char *, struct tm *, const char *, const char *,
+    Imlib_Image);
+static char *scrotGetWindowName(Window);
 static Window scrotGetClientWindow(Display *, Window);
-static Window scrotFindWindowByProperty(Display *, const Window,
-                                              const Atom);
+static Window scrotFindWindowByProperty(Display *, const Window, const Atom);
 static Imlib_Image stalkImageConcat(ScrotList *, const enum Direction);
 static int findWindowManagerFrame(Window *const, int *const);
+static Imlib_Image scrotGrabWindowById(Window const window);
+
+/* X11 stuff */
+Display *disp;
+Window root;
+Window clientWindow;
+Screen *scr;
 
 int main(int argc, char *argv[])
 {
@@ -94,59 +103,67 @@ int main(int argc, char *argv[])
     Imlib_Load_Error imErr;
     char *filenameIM = NULL;
     char *filenameThumb = NULL;
-
     char *haveExtension = NULL;
-
-    time_t t;
+    struct timespec timeStamp;
     struct tm *tm;
 
+    /* Get the time ASAP to reduce the timing error in case --delay is used. */
+    opt.delayStart = clockNow();
+
     atexit(uninitXAndImlib);
 
     optionsParse(argc, argv);
 
     initXAndImlib(opt.display, 0);
 
-    if (!opt.outputFile) {
-        opt.outputFile = estrdup("%Y-%m-%d-%H%M%S_$wx$h_scrot.png");
-        opt.thumbFile = estrdup("%Y-%m-%d-%H%M%S_$wx$h_scrot-thumb.png");
-    } else {
-        if (opt.thumbWorP)
-            opt.thumbFile = optionsNameThumbnail(opt.outputFile);
-        scrotHaveFileExtension(opt.outputFile, &haveExtension);
-    }
-
-    if (opt.focused)
-        image = scrotGrabFocused();
-    else if (opt.selection.mode & SELECTION_MODE_ANY)
+    if (opt.selection.mode & SELECTION_MODE_ANY)
         image = scrotSelectionSelectMode();
-    else if (opt.monitor != -1)
-        image = scrotGrabShotMonitor();
-    else if (opt.autoselect)
-        image = scrotGrabAutoselect();
     else {
+
         scrotDoDelay();
-        if (opt.multidisp)
+
+        if (opt.focused)
+            image = scrotGrabFocused();
+        else if (opt.multidisp)
             image = scrotGrabShotMulti();
         else if (opt.stack)
             image = scrotGrabStackWindows();
+        else if (opt.monitor != -1)
+            image = scrotGrabShotMonitor();
+        else if (opt.autoselect)
+            image = scrotGrabAutoselect();
+        else if (opt.windowId != None)
+            image = scrotGrabWindowById(opt.windowId);
         else
             image = scrotGrabShot();
     }
-
     if (!image)
-        err(EXIT_FAILURE, "no image grabbed");
+        errx(EXIT_FAILURE, "no image grabbed");
+
+    /* Get the time right after the screenshot.
+     * Don't put any new code between this clock_gettime() call and the
+     * screenshot-taking above or it will skew the timing.
+     */
+    clock_gettime(CLOCK_REALTIME, &timeStamp);
+    if (timeStamp.tv_nsec >= miliToNanoSec(500)) {
+        /* Round the timestamp to the nearest second. */
+        timeStamp.tv_sec++;
+    }
+    tm = localtime(&timeStamp.tv_sec);
 
     if (opt.note)
         scrotNoteDraw(image);
 
-    time(&t); /* Get the time directly after the screenshot */
-    tm = localtime(&t);
-
     imlib_context_set_image(image);
+    if (opt.format) {
+        imlib_image_set_format(opt.format);
+    } else {
+        scrotHaveFileExtension(opt.outputFile, &haveExtension);
+        if (!haveExtension)
+            imlib_image_set_format("png");
+    }
     imlib_image_attach_data_value("quality", NULL, opt.quality, NULL);
-
-    if (!haveExtension)
-        imlib_image_set_format("png");
+    imlib_image_attach_data_value("compression", NULL, opt.compression, NULL);
 
     filenameIM = imPrintf(opt.outputFile, tm, NULL, NULL, image);
     scrotCheckIfOverwriteFile(&filenameIM);
@@ -155,32 +172,44 @@ int main(int argc, char *argv[])
 
     imlib_save_image_with_error_return(filenameIM, &imErr);
     if (imErr)
-        err(EXIT_FAILURE, "Saving to file %s failed", filenameIM);
+        errx(EXIT_FAILURE, "Saving to file %s failed", filenameIM);
 
-    if (opt.thumbWorP) {
+    if (opt.thumb != THUMB_DISABLED) {
         int cwidth, cheight;
         int twidth, theight;
 
         cwidth = imlib_image_get_width();
         cheight = imlib_image_get_height();
 
-        if (opt.thumbH) { /* thumbWorP is a width, thumbH is a height. */
-            twidth = opt.thumbWorP;
+        if (opt.thumb == THUMB_RES) {
+            twidth = opt.thumbW;
             theight = opt.thumbH;
-        } else { /* thumbWorP is a percentage, thumbH is unset. */
-            twidth = cwidth * opt.thumbWorP / 100;
-            theight = cheight * opt.thumbWorP / 100;
+            if (twidth == 0)
+                twidth = cwidth * opt.thumbH / cheight;
+            else if (theight == 0)
+                theight = cheight * opt.thumbW / cwidth;
+        } else {
+            twidth = cwidth * opt.thumbPercent / 100;
+            theight = cheight * opt.thumbPercent / 100;
         }
+        /* twidth and theight could be rounded to 0 for extremely small sizes,
+         * fix them up.
+         */
+        if (twidth == 0)
+            twidth = 1;
+        if (theight == 0)
+            theight = 1;
 
         imlib_context_set_anti_alias(1);
         thumbnail = imlib_create_cropped_scaled_image(0, 0, cwidth, cheight,
             twidth, theight);
-        if (!thumbnail)
-            err(EXIT_FAILURE, "unable to create thumbnail");
-        else {
-            scrotHaveFileExtension(opt.thumbFile, &haveExtension);
+        if (!thumbnail) {
+            errx(EXIT_FAILURE, "unable to create thumbnail");
+        } else {
             imlib_context_set_image(thumbnail);
-            if (!haveExtension)
+            if (opt.format)
+                imlib_image_set_format(opt.format);
+            else if (!haveExtension)
                 imlib_image_set_format("png");
 
             filenameThumb = imPrintf(opt.thumbFile, tm, NULL, NULL, thumbnail);
@@ -189,7 +218,7 @@ int main(int argc, char *argv[])
             imlib_free_image_and_decache();
 
             if (imErr)
-                err(EXIT_FAILURE, "Saving thumbnail %s failed", filenameThumb);
+                errx(EXIT_FAILURE, "Saving thumbnail %s failed", filenameThumb);
         }
     }
     if (opt.exec)
@@ -198,10 +227,44 @@ int main(int argc, char *argv[])
     imlib_context_set_image(image);
     imlib_free_image_and_decache();
     free(filenameIM);
+    free(filenameThumb);
 
     return 0;
 }
 
+static void initXAndImlib(const char *dispStr, int screenNumber)
+{
+    disp = XOpenDisplay(dispStr);
+    if (!disp) {
+
+        const char *const format = "Can't open X display. It *is* running, "
+            "yeah? [%s]";
+
+        const char *env = NULL;
+
+        const char *const value = dispStr ? dispStr :
+            (env = getenv("DISPLAY")) ? env : "NULL";
+
+        errx(EXIT_FAILURE, format, value);
+    }
+
+    if (screenNumber)
+        scr = ScreenOfDisplay(disp, screenNumber);
+    else
+        scr = ScreenOfDisplay(disp, DefaultScreen(disp));
+
+    Visual *vis = DefaultVisual(disp, XScreenNumberOfScreen(scr));
+    Colormap cm = DefaultColormap(disp, XScreenNumberOfScreen(scr));
+    root = RootWindow(disp, XScreenNumberOfScreen(scr));
+
+    imlib_context_set_drawable(root);
+    imlib_context_set_display(disp);
+    imlib_context_set_visual(vis);
+    imlib_context_set_colormap(cm);
+    imlib_context_set_color_modifier(NULL);
+    imlib_context_set_operation(IMLIB_OP_COPY);
+}
+
 /* atexit register func. */
 static void uninitXAndImlib(void)
 {
@@ -218,40 +281,45 @@ static void uninitXAndImlib(void)
 
 static size_t scrotHaveFileExtension(const char *filename, char **ext)
 {
-    *ext = strrchr(filename, '.');
+    char *s = strrchr(filename, '.');
 
-    if (*ext)
-        return strlen(*ext);
+    if (s && s[1] != '\0') {
+        *ext = s;
+        return strlen(s);
+    }
 
     return 0;
 }
 
-static Imlib_Image scrotGrabFocused(void)
+static Imlib_Image scrotGrabWindowById(Window const window)
 {
     Imlib_Image im = NULL;
     int rx = 0, ry = 0, rw = 0, rh = 0;
-    Window target = None;
-    int ignored;
 
-    scrotDoDelay();
-    XGetInputFocus(disp, &target, &ignored);
-    if (!scrotGetGeometry(target, &rx, &ry, &rw, &rh))
+    if (!scrotGetGeometry(window, &rx, &ry, &rw, &rh))
         return NULL;
     scrotNiceClip(&rx, &ry, &rw, &rh);
     im = imlib_create_image_from_drawable(0, rx, ry, rw, rh, 1);
     if (opt.pointer)
         scrotGrabMousePointer(im, rx, ry);
-    clientWindow = target;
+    clientWindow = window;
     return im;
 }
 
+static Imlib_Image scrotGrabFocused(void)
+{
+    Window target = None;
+
+    XGetInputFocus(disp, &target, &(int){0});
+    return scrotGrabWindowById(target);
+}
+
 static Imlib_Image scrotGrabAutoselect(void)
 {
     Imlib_Image im = NULL;
     int rx = opt.autoselectX, ry = opt.autoselectY, rw = opt.autoselectW,
         rh = opt.autoselectH;
 
-    scrotDoDelay();
     scrotNiceClip(&rx, &ry, &rw, &rh);
     im = imlib_create_image_from_drawable(0, rx, ry, rw, rh, 1);
     if (opt.pointer)
@@ -261,25 +329,75 @@ static Imlib_Image scrotGrabAutoselect(void)
 
 void scrotDoDelay(void)
 {
-    if (opt.delay) {
-        if (opt.countdown) {
-            int i;
-
-            printf("Taking shot in %d.. ", opt.delay);
-            fflush(stdout);
-            sleep(1);
-            for (i = opt.delay - 1; i > 0; i--) {
-                printf("%d.. ", i);
-                fflush(stdout);
-                sleep(1);
-            }
-            printf("0.\n");
-            fflush(stdout);
-        } else
-            sleep(opt.delay);
+    if (!opt.delay)
+        return;
+    if (opt.countdown) {
+        dprintf(STDERR_FILENO, "Taking shot in ");
+        for (int i = opt.delay; i > 0; i--) {
+            dprintf(STDERR_FILENO, "%d.. ", i);
+            opt.delayStart = scrotSleepFor(opt.delayStart, 1000);
+        }
+        dprintf(STDERR_FILENO, "0.\n");
+    } else {
+        scrotSleepFor(opt.delayStart, opt.delay * 1000);
     }
 }
 
+static long miliToNanoSec(int ms)
+{
+    return ms * 1000L * 1000L;
+}
+
+/* clockNow() has the exact same semantics as CLOCK_MONOTONIC. Except that on
+ * Linux, CLOCK_MONOTONIC does not progress while the system is suspended, so
+ * the non-standard CLOCK_BOOTTIME is used instead to avoid this bug.
+ */
+struct timespec clockNow(void)
+{
+    struct timespec ret;
+#if defined(__linux__)
+    clock_gettime(CLOCK_BOOTTIME, &ret);
+#else
+    clock_gettime(CLOCK_MONOTONIC, &ret);
+#endif
+    return ret;
+}
+
+/* OpenBSD and OS X lack clock_nanosleep(), so we call nanosleep() and use a
+ * trivial algorithm to correct for drift. The end timespec is returned for
+ * callers that want it. EINTR is also dealt with.
+ */
+struct timespec scrotSleepFor(struct timespec start, int ms)
+{
+    scrotAssert(ms >= 0);
+    struct timespec end = {
+        .tv_sec  = start.tv_sec  + (ms / 1000),
+        .tv_nsec = start.tv_nsec + miliToNanoSec(ms % 1000),
+    };
+    if (end.tv_nsec >= miliToNanoSec(1000)) {
+        ++end.tv_sec;
+        end.tv_nsec -= miliToNanoSec(1000);
+    }
+
+    struct timespec tmp;
+    do {
+        tmp = clockNow();
+
+        /* XXX: Use timespecsub(). OS X doesn't have that BSD macro, and libbsd
+         * doesn't support OS X save for an unmaintained fork. libobsd supports
+         * OS X but doesn't have the macro yet.
+         */
+        tmp.tv_sec  = end.tv_sec  - tmp.tv_sec;
+        tmp.tv_nsec = end.tv_nsec - tmp.tv_nsec;
+        if (tmp.tv_nsec < 0) {
+            --tmp.tv_sec;
+            tmp.tv_nsec += miliToNanoSec(1000);
+        }
+    } while (nanosleep(&tmp, NULL) < 0 && errno == EINTR);
+
+    return end;
+}
+
 /* Clip rectangle nicely */
 void scrotNiceClip(int *rx, int *ry, int *rw, int *rh)
 {
@@ -330,24 +448,17 @@ int scrotGetGeometry(Window target, int *rx, int *ry, int *rw, int *rh)
     /* Get windowmanager frame of window */
     if (target != root) {
         if (findWindowManagerFrame(&target, &frames)) {
-
             /* Get client window. */
             if (!opt.border)
                 target = scrotGetClientWindow(disp, target);
+
             XRaiseWindow(disp, target);
+            XSync(disp, False);
 
-            /* Give the WM time to update the hidden area of the window.
-               Some windows never send the event, a time limit is placed.
-            */
-            XSelectInput(disp, target, FocusChangeMask);
-
-            struct timespec delay = {0, 10000000L}; // 10ms
-
-            for (short i = 0; i < 30; ++i) {
-                if (XCheckIfEvent(disp, &(XEvent){0}, &scrotXEventVisibility, (XPointer)&target))
-                    break;
-                nanosleep(&delay, NULL);
-            }
+            /* HACK: there doesn't seem to be any way to figure out whether the
+             * raise request was accepted or rejected. so just sleep a bit to
+             * give the WM some time to update. */
+            scrotSleepFor(clockNow(), 160);
         }
     }
     stat = XGetWindowAttributes(disp, target, &attr);
@@ -358,7 +469,7 @@ int scrotGetGeometry(Window target, int *rx, int *ry, int *rw, int *rh)
     XTranslateCoordinates(disp, target, root, 0, 0, rx, ry, &child);
 
     /* Special case when the TWM emulates the border directly on the window. */
-    if (opt.border == 1 && frames < 2 && attr.border_width > 0) {
+    if (opt.border && frames < 2 && attr.border_width > 0) {
         *rw += attr.border_width * 2;
         *rh += attr.border_width * 2;
         *rx -= attr.border_width;
@@ -394,35 +505,27 @@ Window scrotGetWindow(Display *display, Window window, int x, int y)
     return target;
 }
 
-void scrotGrabMousePointer(const Imlib_Image image, const int xOffset,
+void scrotGrabMousePointer(Imlib_Image image, const int xOffset,
     const int yOffset)
 {
-    XFixesCursorImage *xcim = NULL;
-    uint32_t *pixels = NULL;
-    /* Get the X cursor and the information we want. */
-    if ((xcim = XFixesGetCursorImage(disp)) == NULL) {
-        warnx("Can't get the cursor from X");
-        goto end;
-    }
+    XFixesCursorImage *xcim = XFixesGetCursorImage(disp);
+    if (!xcim)
+        errx(EXIT_FAILURE, "Can't get the cursor from X");
     const unsigned short width = xcim->width;
     const unsigned short height = xcim->height;
     const size_t pixcnt = (size_t)width*height;
 
-    /* xcim->pixels wouldn't be valid if sizeof(*pixels)*pixcnt wrapped around.
-     */
-    if ((pixels = malloc(sizeof(*pixels)*pixcnt)) == NULL) {
-        warn("malloc()");
-        goto end;
+    uint32_t *pixels = (uint32_t *)xcim->pixels;
+    /* XFixesCursorImage returns pixels as `unsigned long`, which is typically
+     * 64bits, but imlib2 expects 32bit packed integers. */
+    if (sizeof *xcim->pixels > sizeof *pixels) {
+        for (size_t i = 0; i < pixcnt; ++i)
+            pixels[i] = xcim->pixels[i];
     }
 
-    /* Convert an X cursor into the format imlib wants. */
-    for (size_t i = 0; i < pixcnt; i++)
-            pixels[i] = xcim->pixels[i];
     Imlib_Image imcursor = imlib_create_image_using_data(width, height, pixels);
-    if (!imcursor) {
-        warnx("Can't create cursor image");
-        goto end;
-    }
+    if (!imcursor)
+        errx(EXIT_FAILURE, "Can't create cursor image");
 
     /* Overlay the cursor into `image`. */
     const int x = (xcim->x - xcim->xhot) - xOffset;
@@ -434,17 +537,17 @@ void scrotGrabMousePointer(const Imlib_Image image, const int xOffset,
         height);
     imlib_context_set_image(imcursor);
     imlib_free_image();
-
-end:
-    free(pixels);
     XFree(xcim);
 }
 
 // It assumes that the local variable 'scrot.c:Imlib_Image image' is in context
 static void applyFilterIfRequired(void)
 {
-    if (opt.script)
+    if (opt.script) {
+        warnx("--script is deprecated. See: "
+            "https://github.com/resurrecting-open-source-projects/scrot/pull/231");
         imlib_apply_filter(opt.script);
+    }
 }
 
 static void scrotCheckIfOverwriteFile(char **filename)
@@ -469,7 +572,7 @@ static void scrotCheckIfOverwriteFile(char **filename)
     if (ext)
         nalloc += extLength; // .ext
 
-    newName = calloc(nalloc, sizeof(*newName));
+    newName = ecalloc(nalloc, sizeof(*newName));
     memcpy(newName, *filename, slen);
 
     do {
@@ -477,7 +580,7 @@ static void scrotCheckIfOverwriteFile(char **filename)
 
         snprintf(fmt, sizeof(fmt), "_%03zu", counter++);
 
-        if(ext) {
+        if (ext) {
             ptr -= extLength;
             memcpy(ptr, fmt, sizeof(fmt));
             memcpy(ptr + sizeof(fmt) - 1, ext, extLength);
@@ -485,7 +588,7 @@ static void scrotCheckIfOverwriteFile(char **filename)
             memcpy(ptr, fmt, sizeof(fmt));
     } while ((counter < maxCounter) && !access(newName, F_OK));
 
-    assert(newName[nalloc - 1] == '\0');
+    scrotAssert(newName[nalloc - 1] == '\0');
 
     free(*filename);
     *filename = newName;
@@ -498,53 +601,20 @@ static void scrotCheckIfOverwriteFile(char **filename)
 
 static int scrotMatchWindowClassName(Window target)
 {
-    assert(disp != NULL);
-
-    const int NOT_MATCH = 0;
-    const int MATCH = 1;
-    /* By default all class names match since windowClassName by default is NULL
-     */
-    int retval = MATCH;
-
+    scrotAssert(disp != NULL);
     if (!opt.windowClassName)
-        return retval;
+        return 1; /* Match any if the user hasn't requested a specific class. */
 
     XClassHint clsHint;
-    retval = NOT_MATCH; // windowClassName != NULL, by default NOT_MATCH
+    int match = 0;
 
     if (XGetClassHint(disp, target, &clsHint) != BadWindow) {
-        retval = optionsCompareWindowClassName(clsHint.res_class);
+        match = strcmp(clsHint.res_class, opt.windowClassName) == 0;
         XFree(clsHint.res_name);
         XFree(clsHint.res_class);
     }
 
-    return retval;
-}
-
-char *scrotGetWindowName(Window window)
-{
-    assert(disp != NULL);
-    assert(window != None);
-
-    if (window == root)
-        return NULL;
-
-    if (!findWindowManagerFrame(&window, &(int){0}))
-        return NULL;
-
-    XClassHint clsHint;
-    char *windowName = NULL;
-
-    const Status status = XGetClassHint(disp,
-            scrotGetClientWindow(disp, window),
-            &clsHint);
-
-    if (status != 0) {
-        windowName = estrdup(clsHint.res_class);
-        XFree(clsHint.res_name);
-        XFree(clsHint.res_class);
-    }
-    return windowName;
+    return match;
 }
 
 static Imlib_Image scrotGrabShot(void)
@@ -565,78 +635,71 @@ static Imlib_Image scrotGrabShot(void)
 static void scrotExecApp(Imlib_Image image, struct tm *tm, char *filenameIM,
     char *filenameThumb)
 {
-    char *execStr;
-    int ret;
-
-    execStr = imPrintf(opt.exec, tm, filenameIM, filenameThumb, image);
-
-    errno = 0;
-
-    ret = system(execStr);
+    char *execStr = imPrintf(opt.exec, tm, filenameIM, filenameThumb, image);
+    int ret = system(execStr);
 
     if (ret == -1)
         warn("The child process could not be created");
-    else if (WEXITSTATUS(ret) == 127)
+    else if (WIFEXITED(ret) && WEXITSTATUS(ret) == 127)
         warnx("scrot could not execute the command: %s", execStr);
-
-    exit(0);
+    free(execStr);
 }
 
-static Bool scrotXEventVisibility(Display *dpy, XEvent *ev, XPointer arg)
+static char *imPrintf(const char *str, struct tm *tm, const char *filenameIM,
+    const char *filenameThumb, Imlib_Image im)
 {
-    (void)dpy; // unused
-    Window *win = (Window *)arg;
-    return (ev->xvisibility.window == *win);
-}
-
-static char *imPrintf(char *str, struct tm *tm, char *filenameIM,
-    char *filenameThumb, Imlib_Image im)
-{
-    char *c;
     char buf[20];
-    char ret[4096];
+    Stream ret = {0};
+    long hostNameMax = 0;
     char strf[4096];
     char *tmp;
     struct stat st;
 
-    ret[0] = '\0';
-
-    if (strftime(strf, 4095, str, tm) == 0)
+    const size_t strfLen = strftime(strf, sizeof(strf), str, tm);
+    if (strfLen == 0)
         errx(EXIT_FAILURE, "strftime returned 0");
 
     imlib_context_set_image(im);
-    for (c = strf; *c != '\0'; c++) {
-        if (*c == '$') {
+    for (const char *c = strf, *end = strf + strfLen; c < end; ++c) {
+        if (*c == '$' && (c + 1) < end) {
             c++;
             switch (*c) {
             case 'a':
-                gethostname(buf, sizeof(buf));
-                strlcat(ret, buf, sizeof(ret));
+                /* freebsd and macos don't have HOST_NAME_MAX defined.
+                 * instead sysconf is recommended by freebsd. */
+                if (hostNameMax == 0)
+                    hostNameMax = sysconf(_SC_HOST_NAME_MAX) + 1; /* +1 for nul-terminator */
+
+                streamReserve(&ret, hostNameMax);
+                char *target = ret.buf + ret.off;
+                gethostname(target, hostNameMax);
+                ret.off += strlen(target);
+                scrotAssert(ret.buf[ret.off] == '\0');
                 break;
             case 'f':
                 if (filenameIM)
-                    strlcat(ret, filenameIM, sizeof(ret));
+                    streamStr(&ret, filenameIM);
                 break;
             case 'm': /* t was already taken, so m as in mini */
                 if (filenameThumb)
-                    strlcat(ret, filenameThumb, sizeof(ret));
+                    streamStr(&ret, filenameThumb);
                 break;
             case 'n':
                 if (filenameIM) {
                     tmp = strrchr(filenameIM, '/');
                     if (tmp)
-                        strlcat(ret, tmp + 1, sizeof(ret));
+                        streamStr(&ret, tmp + 1);
                     else
-                        strlcat(ret, filenameIM, sizeof(ret));
+                        streamStr(&ret, filenameIM);
                 }
                 break;
             case 'w':
                 snprintf(buf, sizeof(buf), "%d", imlib_image_get_width());
-                strlcat(ret, buf, sizeof(ret));
+                streamStr(&ret, buf);
                 break;
             case 'h':
                 snprintf(buf, sizeof(buf), "%d", imlib_image_get_height());
-                strlcat(ret, buf, sizeof(ret));
+                streamStr(&ret, buf);
                 break;
             case 's':
                 if (filenameIM) {
@@ -645,56 +708,77 @@ static char *imPrintf(char *str, struct tm *tm, char *filenameIM,
 
                         size = st.st_size;
                         snprintf(buf, sizeof(buf), "%d", size);
-                        strlcat(ret, buf, sizeof(ret));
+                        streamStr(&ret, buf);
                     } else
-                        strlcat(ret, "[err]", sizeof(ret));
+                        streamStr(&ret, "[err]");
                 }
                 break;
             case 'p':
                 snprintf(buf, sizeof(buf), "%d",
                     imlib_image_get_width() * imlib_image_get_height());
-                strlcat(ret, buf, sizeof(ret));
+                streamStr(&ret, buf);
                 break;
             case 't':
                 tmp = imlib_image_format();
                 if (tmp)
-                    strlcat(ret, tmp, sizeof(ret));
+                    streamStr(&ret, tmp);
                 break;
             case '$':
-                strlcat(ret, "$", sizeof(ret));
+                streamChar(&ret, '$');
                 break;
             case 'W':
-                if (clientWindow) {
-                    if (!(tmp = scrotGetWindowName(clientWindow)))
-                        break;
-                    strlcat(ret, tmp, sizeof(ret));
-                    free(tmp);
+                if (clientWindow && (tmp = scrotGetWindowName(clientWindow))) {
+                    streamStr(&ret, tmp);
+                    XFree(tmp);
                 }
                 break;
             default:
-                snprintf(buf, sizeof(buf), "%.1s", c);
-                strlcat(ret, buf, sizeof(ret));
+                streamChar(&ret, *c);
                 break;
             }
-        } else if (*c == '\\') {
+        } else if (*c == '\\' && (c + 1) < end) {
             c++;
             switch (*c) {
             case 'n':
                 if (filenameIM)
-                    strlcat(ret, "\n", sizeof(ret));
+                    streamChar(&ret, '\n');
                 break;
             default:
-                snprintf(buf, sizeof(buf), "%.1s", c);
-                strlcat(ret, buf, sizeof(ret));
+                streamChar(&ret, *c);
                 break;
             }
         } else {
-            const size_t length = strlen(ret);
-            ret[length] = *c;
-            ret[length + 1] = '\0';
+            streamChar(&ret, *c);
         }
     }
-    return estrdup(ret);
+    streamChar(&ret, '\0');
+    return ret.buf;
+}
+
+/* return value should be freed by XFree() */
+static char *scrotGetWindowName(Window window)
+{
+    scrotAssert(disp != NULL);
+    scrotAssert(window != None);
+
+    if (window == root)
+        return NULL;
+
+    if (!findWindowManagerFrame(&window, &(int){0}))
+        return NULL;
+
+    XClassHint clsHint;
+    char *windowName = NULL;
+
+    const Status status = XGetClassHint(disp,
+            scrotGetClientWindow(disp, window),
+            &clsHint);
+
+    if (status != 0) {
+        windowName = clsHint.res_class;
+        XFree(clsHint.res_name);
+    }
+    return windowName;
 }
 
 static Window scrotGetClientWindow(Display *display, Window target)
@@ -770,8 +854,7 @@ static Imlib_Image scrotGrabStackWindows(void)
     XImage *ximage = NULL;
     XWindowAttributes attr;
     unsigned long i = 0;
-
-#define EWMH_CLIENT_LIST "_NET_CLIENT_LIST" // spec EWMH
+    char EWMH_CLIENT_LIST[] = "_NET_CLIENT_LIST"; // spec EWMH
 
     Atom atomProp = XInternAtom(disp, EWMH_CLIENT_LIST, False);
     Atom atomType = AnyPropertyType;
@@ -861,20 +944,21 @@ static Imlib_Image scrotGrabShotMonitor(void)
     int eventBase, errBase;
 
     if (!XineramaQueryExtension(disp, &eventBase, &errBase))
-        err(EXIT_FAILURE, "Xinerama extension not found");
+        errx(EXIT_FAILURE, "Xinerama extension not found");
 
     int numScreens = 0;
     XineramaScreenInfo *screens = XineramaQueryScreens(disp, &numScreens);
 
     if (!screens && !numScreens)
-        err(EXIT_FAILURE, "Xinerama not active");
+        errx(EXIT_FAILURE, "Xinerama not active");
 
     if (!numScreens)
-        err(EXIT_FAILURE, "Xinerama active but did not find any output device");
+        errx(EXIT_FAILURE, "Xinerama active but did not find any output device");
 
     if (opt.monitor >= numScreens)
-        err(EXIT_FAILURE, "monitor %d not found", opt.monitor);
+        errx(EXIT_FAILURE, "monitor %d not found", opt.monitor);
 
+    scrotAssert(screens); /* silence clang-tidy */
     XineramaScreenInfo *mon = &screens[opt.monitor];
 
     /* Hack: pretend we were invoked in autoselect mode */
