@@ -1,7 +1,8 @@
 /* scrot.h
 
-Copyright 2021 Guilherme Janczak <guilherme.janczak@yandex.com>
-Copyright 2022 Daniel T. Borelli <danieltborelli@gmail.com>
+Copyright 2021-2023 Guilherme Janczak <guilherme.janczak@yandex.com>
+Copyright 2022      Daniel T. Borelli <danieltborelli@gmail.com>
+Copyright 2023      NRK <nrk@disroot.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -26,14 +27,25 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 /* Part of the code comes from the scrot.c file and maintains its authorship. */
 
-#pragma once
+#ifndef H_SCROT
+#define H_SCROT
+
+#include <time.h>
 
 #include <X11/Xlib.h>
 #include <Imlib2.h>
 
+extern Display *disp;
+extern Window root;
+extern Window clientWindow;
+extern Screen *scr;
+
 Window scrotGetWindow(Display *, Window, int, int);
 int scrotGetGeometry(Window, int *, int *, int *, int *);
 void scrotNiceClip(int *, int *, int *, int *);
+struct timespec clockNow(void);
+struct timespec scrotSleepFor(struct timespec, int);
 void scrotDoDelay(void);
 void scrotGrabMousePointer(const Imlib_Image, const int, const int);
-char *scrotGetWindowName(Window);
+
+#endif /* !defined(H_SCROT) */
