@@ -5,7 +5,7 @@ Copyright 2021-2023 Guilherme Janczak <guilherme.janczak@yandex.com>
 Copyright 2021      Martin C <martincation@protonmail.com>
 Copyright 2021      Peter Wu <peterwu@hotmail.com>
 Copyright 2021      Wilson Smith <01wsmith+gh@gmail.com>
-Copyright 2022      NRK <nrk@disroot.org>
+Copyright 2022-2023 NRK <nrk@disroot.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -33,9 +33,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
     Part of the code comes from the scrot.c file and maintains its authorship.
 */
 
-#include <sys/select.h>
-
-#include <assert.h>
 #include <err.h>
 #include <errno.h>
 #include <stdint.h>
@@ -48,12 +45,12 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
 
-#include "imlib.h"
 #include "options.h"
 #include "scrot.h"
 #include "scrot_selection.h"
 #include "selection_classic.h"
 #include "selection_edge.h"
+#include "util.h"
 
 struct Selection **selectionGet(void)
 {
@@ -64,7 +61,7 @@ struct Selection **selectionGet(void)
 static void selectionAllocate(void)
 {
     struct Selection **sel = selectionGet();
-    *sel = calloc(1, sizeof(**sel));
+    *sel = ecalloc(1, sizeof(**sel));
 }
 
 static void selectionDeallocate(void)
@@ -78,8 +75,6 @@ static void createCursors(void)
 {
     struct Selection *const sel = *selectionGet();
 
-    assert(sel != NULL);
-
     if (opt.selection.mode == SELECTION_MODE_CAPTURE)
         sel->curCross = XCreateFontCursor(disp, XC_cross);
     else if (opt.selection.mode == SELECTION_MODE_HIDE)
@@ -97,8 +92,7 @@ static void createCursors(void)
 static void freeCursors(void)
 {
     struct Selection *const sel = *selectionGet();
-
-    assert(sel != NULL);
+    scrotAssert(sel); /* silence clang-tidy */
 
     XFreeCursor(disp, sel->curCross);
     XFreeCursor(disp, sel->curAngleNE);
@@ -139,23 +133,21 @@ void scrotSelectionCreate(void)
 
     struct Selection *const sel = *selectionGet();
 
-    assert(sel != NULL);
-
     createCursors();
 
-    if (!strncmp(opt.lineMode, LINE_MODE_S_CLASSIC, LINE_MODE_L_CLASSIC)) {
+    if (opt.lineMode == LINE_MODE_CLASSIC) {
         sel->create = selectionClassicCreate;
         sel->draw = selectionClassicDraw;
         sel->motionDraw = selectionClassicMotionDraw;
         sel->destroy = selectionClassicDestroy;
-    } else if (!strncmp(opt.lineMode, LINE_MODE_S_EDGE, LINE_MODE_L_EDGE)) {
+    } else if (opt.lineMode == LINE_MODE_EDGE) {
         sel->create = selectionEdgeCreate;
         sel->draw = selectionEdgeDraw;
         sel->motionDraw = selectionEdgeMotionDraw;
         sel->destroy = selectionEdgeDestroy;
     } else {
         // It never happened, fix the options.c file
-        assert(0);
+        scrotAssert(0);
     }
 
     sel->create();
@@ -211,8 +203,8 @@ struct SelectionRect *scrotSelectionGetRect(void)
 
 Status scrotSelectionCreateNamedColor(const char *nameColor, XColor *color)
 {
-    assert(nameColor != NULL);
-    assert(color != NULL);
+    scrotAssert(nameColor != NULL);
+    scrotAssert(color != NULL);
 
     return XAllocNamedColor(disp, XDefaultColormap(disp, DefaultScreen(disp)),
         nameColor, color, &(XColor){0});
@@ -220,44 +212,31 @@ Status scrotSelectionCreateNamedColor(const char *nameColor, XColor *color)
 
 void scrotSelectionGetLineColor(XColor *color)
 {
-    scrotSelectionSetDefaultColorLine();
-
     const Status ret = scrotSelectionCreateNamedColor(opt.lineColor, color);
 
     if (!ret) {
         scrotSelectionDestroy();
-        errx(EXIT_FAILURE, "Error allocate color:%s", strerror(BadColor));
+        errx(EXIT_FAILURE, "Error allocating color: %s", opt.lineColor);
     }
 }
 
-void scrotSelectionSetDefaultColorLine(void)
-{
-    if (!opt.lineColor)
-        opt.lineColor = "gray";
-}
-
 bool scrotSelectionGetUserSel(struct SelectionRect *selectionRect)
 {
-    static int xfd = 0;
-    static int fdSize = 0;
     XEvent ev;
-    fd_set fdSet;
-    int count = 0, done = 0;
-    int rx = 0, ry = 0, rw = 0, rh = 0, isButtonPressed = 0;
+    enum { WAIT, DONE, ABORT } done = WAIT;
+    int rx = 0, ry = 0, rw = 0, rh = 0;
+    bool isButtonPressed = false;
     Window target = None;
     Status ret;
 
     scrotSelectionCreate();
 
-    xfd = ConnectionNumber(disp);
-    fdSize = xfd + 1;
-
     ret = XGrabKeyboard(disp, root, False, GrabModeAsync, GrabModeAsync, CurrentTime);
     if (ret == AlreadyGrabbed) {
         int attempts = 20;
-        struct timespec delay = { 0, 50 * 1000L * 1000L };
+        struct timespec t = clockNow();
         do {
-            nanosleep(&delay, NULL);
+            t = scrotSleepFor(t, 50);
             ret = XGrabKeyboard(disp, root, False, GrabModeAsync, GrabModeAsync, CurrentTime);
         } while (--attempts > 0 && ret == AlreadyGrabbed);
     }
@@ -266,89 +245,74 @@ bool scrotSelectionGetUserSel(struct SelectionRect *selectionRect)
         errx(EXIT_FAILURE, "failed to grab keyboard");
     }
 
-    while (1) {
-        /* Handle events here */
-        while (!done && XPending(disp)) {
-            XNextEvent(disp, &ev);
-            switch (ev.type) {
-            case MotionNotify:
-                if (isButtonPressed)
-                    scrotSelectionMotionDraw(rx, ry, ev.xbutton.x, ev.xbutton.y);
+    while (done == WAIT) {
+        XNextEvent(disp, &ev);
+        switch (ev.type) {
+        case MotionNotify:
+            if (isButtonPressed)
+                scrotSelectionMotionDraw(rx, ry, ev.xbutton.x, ev.xbutton.y);
+            break;
+        case ButtonPress:
+            isButtonPressed = true;
+            rx = ev.xbutton.x;
+            ry = ev.xbutton.y;
+            target = scrotGetWindow(disp, ev.xbutton.subwindow, ev.xbutton.x, ev.xbutton.y);
+            if (target == None)
+                target = root;
+            break;
+        case ButtonRelease:
+            done = DONE;
+            break;
+        case KeyPress:
+        {
+            KeySym *keysym = NULL;
+            int keycode; /*dummy*/
+
+            keysym = XGetKeyboardMapping(disp, ev.xkey.keycode, 1, &keycode);
+
+            if (!keysym)
                 break;
-            case ButtonPress:
-                isButtonPressed = 1;
-                rx = ev.xbutton.x;
-                ry = ev.xbutton.y;
-                target = scrotGetWindow(disp, ev.xbutton.subwindow, ev.xbutton.x, ev.xbutton.y);
-                if (target == None)
-                    target = root;
-                break;
-            case ButtonRelease:
-                done = 1;
-                break;
-            case KeyPress:
-            {
-                KeySym *keysym = NULL;
-                int keycode; /*dummy*/
 
-                keysym = XGetKeyboardMapping(disp, ev.xkey.keycode, 1, &keycode);
-
-                if (!keysym)
-                    break;
-
-                if (!isButtonPressed) {
-                key_abort_shot:
-                    if (!opt.ignoreKeyboard || *keysym == XK_Escape) {
-                        warnx("Key was pressed, aborting shot");
-                        done = 2;
-                    }
-                    XFree(keysym);
-                    break;
-                }
-
-                switch (*keysym) {
-                case XK_Right:
-                    if (++rx > scr->width)
-                        rx = scr->width;
-                    break;
-                case XK_Left:
-                    if (--rx < 0)
-                        rx = 0;
-                    break;
-                case XK_Down:
-                    if (++ry > scr->height)
-                        ry = scr->height;
-                    break;
-                case XK_Up:
-                    if (--ry < 0)
-                        ry = 0;
-                    break;
-                default:
-                    goto key_abort_shot;
+            if (!isButtonPressed) {
+            key_abort_shot:
+                if (!opt.ignoreKeyboard || *keysym == XK_Escape) {
+                    warnx("Key was pressed, aborting shot");
+                    done = ABORT;
                 }
                 XFree(keysym);
-                scrotSelectionMotionDraw(rx, ry, ev.xbutton.x, ev.xbutton.y);
                 break;
             }
-            case KeyRelease:
-                /* ignore */
+
+            switch (*keysym) {
+            case XK_Right:
+                if (++rx > scr->width)
+                    rx = scr->width;
+                break;
+            case XK_Left:
+                if (--rx < 0)
+                    rx = 0;
+                break;
+            case XK_Down:
+                if (++ry > scr->height)
+                    ry = scr->height;
+                break;
+            case XK_Up:
+                if (--ry < 0)
+                    ry = 0;
                 break;
             default:
-                break;
+                goto key_abort_shot;
             }
+            XFree(keysym);
+            scrotSelectionMotionDraw(rx, ry, ev.xbutton.x, ev.xbutton.y);
+            break;
         }
-        if (done)
+        case DestroyNotify:
+            errx(EXIT_FAILURE, "received DestroyNotify event");
+            break;
+        default:
+            /* ignore */
             break;
-
-        /* now block some */
-        FD_ZERO(&fdSet);
-        FD_SET(xfd, &fdSet);
-        errno = 0;
-        count = select(fdSize, &fdSet, NULL, NULL, NULL);
-        if ((count < 0)
-            && ((errno == ENOMEM) || (errno == EINVAL) || (errno == EBADF))) {
-            scrotSelectionDestroy();
-            errx(EXIT_FAILURE, "Connection to X display lost");
         }
     }
     scrotSelectionDraw();
@@ -359,7 +323,7 @@ bool scrotSelectionGetUserSel(struct SelectionRect *selectionRect)
 
     scrotSelectionDestroy();
 
-    if (done == 2)
+    if (done == ABORT)
         return false;
 
     if (isAreaSelect) {
@@ -386,7 +350,7 @@ bool scrotSelectionGetUserSel(struct SelectionRect *selectionRect)
         // Not record pointer if there is a selection area because it is busy on that,
         // unless the delay option is used.
         if (opt.delay == 0)
-            opt.pointer = 0;
+            opt.pointer = false;
     } else {
         /* else it's a window click */
         if (!scrotGetGeometry(target, &rx, &ry, &rw, &rh))
@@ -457,6 +421,9 @@ Imlib_Image scrotSelectionSelectMode(void)
 
     opt.selection.mode = SELECTION_MODE_CAPTURE;
 
+    if (opt.delaySelection)
+        scrotDoDelay();
+
     if (!scrotSelectionGetUserSel(&rect0))
         return NULL;
 
@@ -466,7 +433,10 @@ Imlib_Image scrotSelectionSelectMode(void)
         if (!scrotSelectionGetUserSel(&rect1))
             return NULL;
 
-    scrotDoDelay();
+    if (!opt.delaySelection) {
+        opt.delayStart = clockNow();
+        scrotDoDelay();
+    }
 
     Imlib_Image capture = imlib_create_image_from_drawable(0, rect0.x, rect0.y,
         rect0.w, rect0.h, 1);
@@ -480,13 +450,13 @@ Imlib_Image scrotSelectionSelectMode(void)
     XColor color;
     scrotSelectionGetLineColor(&color);
 
-    const int x = rect1.x - rect0.x;
+    const int x = rect1.x - rect0.x; /* NOLINT(*UndefinedBinaryOperatorResult) */
     const int y = rect1.y - rect0.y;
     const int opacity = opt.lineOpacity;
 
     imlib_context_set_image(capture);
 
-    switch(opt.selection.mode) {
+    switch (opt.selection.mode) {
     case SELECTION_MODE_HOLE:
         if (opacity > 0) {
             Imlib_Image hole = imlib_clone_image();
@@ -499,7 +469,7 @@ Imlib_Image scrotSelectionSelectMode(void)
         break;
     case SELECTION_MODE_HIDE:
     {
-        char *const fileName = opt.selection.paramStr;
+        const char *const fileName = opt.selection.fileName;
 
         if (fileName) {
             if (opacity > 0) {
@@ -512,7 +482,6 @@ Imlib_Image scrotSelectionSelectMode(void)
                 imlib_context_set_image(hide);
                 imlib_free_image_and_decache();
             }
-            free(fileName);
         } else {
             imlib_context_set_color(color.red, color.green, color.blue, opacity);
             imlib_image_fill_rectangle(x, y, rect1.w, rect1.h);
@@ -521,7 +490,7 @@ Imlib_Image scrotSelectionSelectMode(void)
     }
     case SELECTION_MODE_BLUR:
     {
-        const int amountBlur = opt.selection.paramNum;
+        const int amountBlur = opt.selection.blur;
         Imlib_Image blur = imlib_clone_image();
         imlib_context_set_image(blur);
         imlib_image_blur(amountBlur);
@@ -532,7 +501,7 @@ Imlib_Image scrotSelectionSelectMode(void)
         break;
     }
     default:
-        assert(0);
+        scrotAssert(0 && "unreachable");
     }
 
     return capture;
