@@ -4,6 +4,7 @@ Copyright 2020-2021 Daniel T. Borelli <danieltborelli@gmail.com>
 Copyright 2021-2023 Guilherme Janczak <guilherme.janczak@yandex.com>
 Copyright 2021      Martin C <martincation@protonmail.com>
 Copyright 2021      Peter Wu <peterwu@hotmail.com>
+Copyright 2023      NRK <nrk@disroot.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -31,7 +32,8 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
     Part of the code comes from the scrot.c file and maintains its authorship.
 */
 
-#pragma once
+#ifndef H_SCROT_SELECTION
+#define H_SCROT_SELECTION
 
 #include <stdbool.h>
 
@@ -39,10 +41,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <X11/Xlib.h>
 
 /* S: string, L: len */
-#define LINE_MODE_S_CLASSIC "classic"
-#define LINE_MODE_L_CLASSIC 7
-#define LINE_MODE_S_EDGE "edge"
-#define LINE_MODE_L_EDGE 4
 #define SELECTION_MODE_S_CAPTURE "capture"
 #define SELECTION_MODE_L_CAPTURE 7
 #define SELECTION_MODE_S_HIDE "hide"
@@ -52,6 +50,11 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #define SELECTION_MODE_S_BLUR "blur"
 #define SELECTION_MODE_L_BLUR 4
 
+enum LineMode {
+    LINE_MODE_CLASSIC,
+    LINE_MODE_EDGE,
+};
+
 enum {
     SELECTION_MODE_SEPARATOR = ',',
     SELECTION_MODE_CAPTURE = (1 << 1),
@@ -76,8 +79,8 @@ struct SelectionRect {
 
 typedef struct SelectionMode {
     unsigned int mode;
-    int paramNum;
-    char *paramStr;
+    int blur;
+    const char *fileName;
 } SelectionMode;
 
 struct SelectionClassic;
@@ -108,3 +111,5 @@ Status scrotSelectionCreateNamedColor(const char *, XColor *);
 void scrotSelectionSetDefaultColorLine(void);
 bool scrotSelectionGetUserSel(struct SelectionRect *);
 Imlib_Image scrotSelectionSelectMode(void);
+
+#endif /* !defined(H_SCROT_SELECTION) */
