@@ -1,8 +1,9 @@
 /* scrot_selection_classic.c
 
 Copyright 2020-2021 Daniel T. Borelli <danieltborelli@gmail.com>
-Copyright 2021-2022 Guilherme Janczak <guilherme.janczak@yandex.com>
+Copyright 2021-2023 Guilherme Janczak <guilherme.janczak@yandex.com>
 Copyright 2021      Peter Wu <peterwu@hotmail.com>
+Copyright 2023      NRK <nrk@disroot.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -30,17 +31,17 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
     Part of the code comes from the scrot.c file and maintains its authorship.
 */
 
-#include <assert.h>
 #include <stdlib.h>
+#include <err.h>
 
 #include <X11/Xlib.h>
 
-#include "imlib.h"
 #include "options.h"
 #include "scrot.h"
 #include "scrot_selection.h"
 #include "selection_classic.h"
 #include "selection_edge.h"
+#include "util.h"
 
 struct SelectionClassic {
     XGCValues gcValues;
@@ -51,7 +52,7 @@ void selectionClassicCreate(void)
 {
     struct Selection *const sel = *selectionGet();
 
-    sel->classic = calloc(1, sizeof(*sel->classic));
+    sel->classic = ecalloc(1, sizeof(*sel->classic));
 
     struct SelectionClassic *pc = sel->classic;
 
@@ -73,8 +74,8 @@ void selectionClassicCreate(void)
     pc->gc = XCreateGC(disp, root,
         GCFunction | GCForeground | GCBackground | GCSubwindowMode,
         &pc->gcValues);
-
-    assert(pc->gc != NULL);
+    if (pc->gc == NULL)
+        errx(EXIT_FAILURE, "failed to create GC");
 
     XSetLineAttributes(disp, pc->gc, opt.lineWidth, opt.lineStyle, CapRound,
         JoinRound);
