@@ -3,6 +3,7 @@
 Copyright 2020-2021 Daniel T. Borelli <danieltborelli@gmail.com>
 Copyright 2021-2023 Guilherme Janczak <guilherme.janczak@yandex.com>
 Copyright 2021      Peter Wu <peterwu@hotmail.com>
+Copyright 2023      NRK <nrk@disroot.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -30,6 +31,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
     Part of the code comes from the scrot.c file and maintains its authorship.
 */
 
+#include <errno.h>
 #include <stdlib.h>
 #include <time.h>
 
@@ -38,24 +40,22 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <X11/Xutil.h>
 #include <X11/extensions/shape.h>
 
-#include "imlib.h"
 #include "options.h"
+#include "scrot.h"
 #include "scrot_selection.h"
 #include "selection_edge.h"
+#include "util.h"
 
 struct SelectionEdge {
     Window wndDraw;
     XClassHint *classHint;
 };
 
-static void waitUnmapWindowNotify(void);
-static Bool xeventUnmap(Display *, XEvent *, XPointer);
-
 void selectionEdgeCreate(void)
 {
     struct Selection *const sel = *selectionGet();
 
-    sel->edge = calloc(1, sizeof(*sel->edge));
+    sel->edge = ecalloc(1, sizeof(*sel->edge));
 
     struct SelectionEdge *const pe = sel->edge;
 
@@ -128,38 +128,25 @@ void selectionEdgeDestroy(void)
     const struct Selection *const sel = *selectionGet();
     struct SelectionEdge *pe = sel->edge;
 
-    if (pe->wndDraw != 0) {
-        waitUnmapWindowNotify();
-        XFree(pe->classHint);
+    if (pe->wndDraw != None) {
+        XSelectInput(disp, pe->wndDraw, StructureNotifyMask);
         XDestroyWindow(disp, pe->wndDraw);
+        bool is_unmapped = false, is_destroyed = false;
+        for (XEvent ev; !(is_unmapped && is_destroyed);) {
+            XNextEvent(disp, &ev);
+            if (ev.type == DestroyNotify && ev.xdestroywindow.window == pe->wndDraw)
+                is_destroyed = true;
+            if (ev.type == UnmapNotify && ev.xunmap.window == pe->wndDraw)
+                is_unmapped = true;
+        }
+        /* HACK: although we recived a DestroyNotify event, the frame still
+         * might not have been updated. a compositor might also buffer frames
+         * adding latency. so wait a bit for the screen to update and the
+         * selection borders to go away. */
+        scrotSleepFor(clockNow(), 80);
+
+        XFree(pe->classHint);
     }
 
     free(pe);
 }
-
-static void waitUnmapWindowNotify(void)
-{
-    const struct Selection *const sel = *selectionGet();
-    const struct SelectionEdge *const pe = sel->edge;
-    XEvent ev;
-
-    XSelectInput(disp, pe->wndDraw, StructureNotifyMask);
-
-    XUnmapWindow(disp, pe->wndDraw);
-
-    struct timespec delay = {0, 80000000L}; // 80ms
-
-    for (short i = 0; i < 30; ++i) {
-        if (XCheckIfEvent(disp, &ev, &xeventUnmap, (XPointer) & (pe->wndDraw)))
-            break;
-        nanosleep(&delay, NULL);
-    }
-}
-
-
-static Bool xeventUnmap(Display *dpy, XEvent *ev, XPointer arg)
-{
-    (void)dpy; // unused
-    Window *win = (Window *)arg;
-    return (ev->xunmap.window == *win);
-}
