@@ -1,6 +1,7 @@
 /* util.h
 
-Copyright 2021 Guilherme Janczak <guilherme.janczak@yandex.com>
+Copyright 2021-2023 Guilherme Janczak <guilherme.janczak@yandex.com>
+Copyright 2023      NRK <nrk@disroot.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -23,6 +24,38 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 */
 
-#pragma once
+#ifndef H_UTIL
+#define H_UTIL
+
+#include <stdio.h>
+#include <stdlib.h>
+
+#ifdef DEBUG
+    #define scrotAssert(X) do { \
+        if (!(X)) { \
+            fprintf(stderr, "%s:%d: `%s`\n", __FILE__, __LINE__, #X); \
+            abort(); \
+        } \
+    } while (0)
+#else
+    #define scrotAssert(X) ((void)0)
+#endif
+
+#define ARRAY_COUNT(X)   (sizeof(X) / sizeof(0[X]))
+#define MAX(A, B)        ((A) > (B) ? (A) : (B))
+
+typedef struct {
+    char *buf;
+    size_t off, cap;
+} Stream;
 
 char *estrdup(const char *);
+void *ecalloc(size_t, size_t);
+void *erealloc(void *, size_t);
+
+void streamReserve(Stream *, size_t);
+void streamChar(Stream *, char);
+void streamMem(Stream *, const void *, size_t);
+void streamStr(Stream *, const char *);
+
+#endif /* !defined(H_UTIL) */
